import glob
import json
import os
import re

from lxml import etree
import pytest

import inventory_provider
from inventory_provider import juniper

TEST_DATA_DIRNAME = os.path.realpath(os.path.join(
    inventory_provider.__path__[0],
    "..",
    "test",
    "data"))


@pytest.fixture
def classifier_cache_test_entries():
    filename = os.path.join(
        TEST_DATA_DIRNAME, 'classifier-cache-entries.json')
    with open(filename) as f:
        return json.loads(f.read())


def pytest_generate_tests(metafunc):
    # TODO: can we really not get netconf data for all routers?
    def _available_netconf_hosts():
        for fn in glob.glob(os.path.join(TEST_DATA_DIRNAME, '*-netconf.xml')):
            m = re.match('(.*)-netconf.xml', os.path.basename(fn))
            assert m  # sanity
            yield m.group(1)

    routers = list(_available_netconf_hosts())
    metafunc.parametrize("router", routers)


class MockedJunosRpc(object):

    def __init__(self, hostname):
        filename = os.path.join(TEST_DATA_DIRNAME, "%s-netconf.xml" % hostname)
        self.config = etree.parse(filename)

    def get_config(self):
        return self.config


class MockedJunosDevice(object):

    def __init__(self, **kwargs):
        self.rpc = MockedJunosRpc(kwargs['host'])

    def open(self):
        pass


@pytest.fixture
def netconf_doc(mocker, router, data_config):
    mocker.patch(
        'inventory_provider.juniper.Device',
        MockedJunosDevice)
    return juniper.load_config(router, data_config['ssh'])
